@@ -1,9 +1,11 @@
 #!/usr/bin/env python3
 
-# SPDX-FileCopyrightText: 2018-2021, Marsiske Stefan
+# SPDX-FileCopyrightText: 2018-2024, Marsiske Stefan
 # SPDX-License-Identifier: GPL-3.0-or-later
 
 import socket, sys, ssl, os, datetime, binascii, shutil, os.path, traceback, struct
+from os import access, R_OK
+from os.path import isfile, getsize
 import pysodium
 import equihash
 import pyoprf
@@ -11,6 +13,7 @@ from pwdsphinx.config import getcfg
 from pwdsphinx.consts import *
 cfg = getcfg('sphinx')
 
+# todo genkey ltsig integrate into oracle
 
 verbose = cfg['server'].get('verbose', False)
 address = cfg['server'].get('address', '127.0.0.1')
@@ -19,7 +22,6 @@ timeout = int(cfg['server'].get('timeout',"3"))
 max_kids = int(cfg['server'].get('max_kids',5))
 datadir = os.path.expanduser(cfg['server'].get('datadir',"/var/lib/sphinx"))
 ts_epsilon = 1200 # todo make configurable
-debug = False # todo make configurable
 try:
     ssl_key = os.path.expanduser(cfg['server']['ssl_key'])
 except KeyError:
@@ -77,7 +79,7 @@ def fail(s):
     if verbose:
         traceback.print_stack()
         print('fail')
-    s.send(b'\x00\x04fail') # plaintext :/
+    s.send(b'\x00\x04fail') # plaintext :/ todo use ltsigkey?
     s.shutdown(socket.SHUT_RDWR)
     s.close()
     os._exit(0)
@@ -185,7 +187,7 @@ def create(s, msg):
     # 1st step OPRF with a new seed
     k=b'\x01'+pysodium.randombytes(32)
     try:
-      beta = pyoprf.evaluate(k[1:], alpha)
+      beta = k[:1]+pyoprf.evaluate(k[1:], alpha)
     except:
       fail(s)
 
@@ -365,7 +367,7 @@ def auth(s,id,alpha):
     print('bad sig')
     fail(s)
   else:
-    s.send(b'\x00\x04auth') # plaintext :/
+    s.send(b'\x00\x04auth') # plaintext :/ todo use ltsigkey?
 
 def change(conn, msg):
   op,   msg = pop(msg,1)
@@ -390,7 +392,7 @@ def change(conn, msg):
   k=b'\x01'+pysodium.randombytes(32)
 
   try:
-      beta = pyoprf.evaluate(k[1:], alpha)
+      beta = k[:1]+pyoprf.evaluate(k[1:], alpha)
   except:
     fail(conn)
 
@@ -696,7 +698,7 @@ def ratelimit(conn):
    elif op == CHALLENGE_VERIFY:
      verify_challenge(conn)
 
-def main():
+def main(debug=False):
     if debug == True:
         import ctypes
         libc = ctypes.cdll.LoadLibrary('libc.so.6')
@@ -771,5 +773,47 @@ def main():
         pass
     s.close()
 
+def is_readable(path):
+  return isfile(path) and access(path, R_OK)
+
+def missing_file(path):
+    print(f"The SSL key at {path} is not a readable file. Make sure this is a proper ssl key.")
+    print(f"Our GettingStarted document gives simple example of how to do so.")
+    print(f"Check out https://sphinx.pm/server_install.html .")
+    print(f"Aborting.")
+    exit(1)
+
+def parse_params():
+  if not is_readable(ssl_key):
+    missing_file(ssl_key)
+  if not is_readable(ssl_cert):
+    missing_file(ssl_cert)
+  if not is_readable(ltsigkey_path) and 'init' not in sys.argv:
+    print(f"Long-term signing key at {ltsigkey_path} is not readable.")
+    print(f"You can generate one by running: {sys.argv[0]} init")
+  if not getsize != pysodium.crypto_sign_SECRETKEYBYTES:
+    print(f"The long-term signing key of the oracle has an invalid size, maybe it's corrupt?")
+    print("abort.")
+    exit(1)
+
+  debug=False
+  if 'debug' in sys.argv:
+    debug = True
+  if not 'init' in sys.argv:
+      main(debug)
+
+  # init
+  pk, sk = pysodium.crypto_sign_keypair()
+  with open(ltsigkey_path, 'xb') as fd:
+    fd.write(sk)
+
+  with open(f"{ltsigkey_path}.pub", 'xb') as fd:
+    fd.write(pk)
+
+  print(f"successfully created long-term signature key pair at:")
+  print(f"{ltsigkey_path}")
+  print(f"and the public key - which you should make available to all clients -, is at:")
+  print(f"{ltsigkey_path}.pub")
+
 if __name__ == '__main__':
-  main()
+  parse_params()
