@@ -1,9 +1,9 @@
 #!/usr/bin/env python3
 
-# SPDX-FileCopyrightText: 2018-2021, Marsiske Stefan
+# SPDX-FileCopyrightText: 2018-2024, Marsiske Stefan
 # SPDX-License-Identifier: GPL-3.0-or-later
 
-import sys, os, socket, ssl, struct, platform, getpass, binascii
+import sys, os, socket, ssl, struct, platform, getpass, binascii, zlib
 import concurrent.futures
 from SecureString import clearmem
 import pysodium, pyoprf
@@ -181,15 +181,15 @@ def unpack_rule(ct):
 
   return rule, symbols, size, check_digit, xor_mask
 
-def pack_rule(char_classes, syms, size, check_digit, xor_mask=None):
+def pack_rule(char_classes, syms, size, check_digit, xor_mask=None, with_schema=False):
   # pack rules into and encrypt them
   if set(char_classes) - {'u','l','d'}:
     raise ValueError("ERROR: rules can only contain any of 'uld'.")
   if set(syms) - set(bin2pass.symbols) != set():
     raise ValueError("ERROR: symbols can only contain any of '%s'." % bin2pass.symbols)
-  if char_classes == '' and len(syms)<2:
+  if not with_schema and (char_classes == '' and len(syms)<2):
     raise ValueError("ERROR: no char classes and not enough symbols specified.")
-  if xor_mask is None and (char_classes == '' and len(syms)<2):
+  if not with_schema and (xor_mask is None and (char_classes == '' and len(syms)<2)):
     raise ValueError("ERROR: no char classes and not enough symbols specified.")
   if xor_mask is None:
       xor_mask = b'\x00' * 32
@@ -291,6 +291,8 @@ def auth(m,ids,alpha=None,pwd=None,r=None):
       m.close()
       return False
     nonces = [(idx,resp[1]) for idx, resp in enumerate(msgs)]
+    if len(msgs) < len(m):
+        raise ValueError("auth: not all peers answered or authenticated")
     beta = pyoprf.thresholdmult([resp[0] for resp in msgs][:threshold])
     rwd = pyoprf.unblind_finalize(r, beta, pwd)
 
@@ -315,6 +317,9 @@ def auth(m,ids,alpha=None,pwd=None,r=None):
     else:
       raise ValueError("unexpected auth result")
 
+  if fails > 0:
+      raise ValueError("some peers have failed to authenticate us")
+
   return rwd
 
 def ratelimit(m,reqs):
@@ -322,14 +327,10 @@ def ratelimit(m,reqs):
     pkt0 = b''.join([CHALLENGE_CREATE, req])
     m[i].send(pkt0)
 
-  #challenge = s.recv(1+1+8+32) # n,k,ts,sig
-  challenges = m.gather(1+1+8+32)
+  challenges = m.gather(1+1+8+32) # n,k,ts,sig
   if challenges is None:
     m.close()
     return False
-  #if len(challenge)!= 1+1+8+32:
-  #  if verbose: print("challengelen incorrect: %s %s" %(len(challenge), repr(challenge)), file=sys.stderr)
-  #  raise ValueError("ERROR: failed to get ratelimit challenge")
   m.close()
 
   puzzles = []
@@ -465,18 +466,18 @@ def create(m, pwd, user, host, char_classes='uld', symbols=bin2pass.symbols, siz
   if threshold > 1:
     beta = dkg(m, CREATE_DKG, threshold, ids, alpha)
   else:
-    msg = b''.join([CREATE, id[0], alpha])
+    msg = b''.join([CREATE, ids[0], alpha])
     m.broadcast(msg)
 
     # wait for response from sphinx server
-    beta = m.gather(32)
+    beta = m.gather(33)
     if beta is None:
       m.close()
       raise ValueError("ERROR: Creating new password, the record probably already exists or the first message to server was corrupted during transport.")
       # or (less probable) the initial message was longer/shorter than the 65 bytes we sent
       # or (even? less probable) the value alpha received by the server is not a valid point
       # both of these less probable causes point at corruption during transport
-    beta = beta[0]
+    beta = beta[0][1:]
 
   rwd = pyoprf.unblind_finalize(r, beta, pwd)
 
@@ -520,7 +521,7 @@ def create(m, pwd, user, host, char_classes='uld', symbols=bin2pass.symbols, siz
             continue
         break
 
-  rule = pack_rule(char_classes, symbols, size, checkdigit, xormask)
+  rule = pack_rule(char_classes, symbols, size, checkdigit, xormask, with_schema='://' in user)
   # send over new signed(pubkey, rule)
   for i,(id,pk) in enumerate(zip(ids, sign_keys)):
     msg = b''.join([pk, rule])
@@ -540,7 +541,7 @@ def create(m, pwd, user, host, char_classes='uld', symbols=bin2pass.symbols, siz
   clearmem(rwd)
   return ret
 
-def get(m, pwd, user, host, raw=False):
+def get(m, pwd, user, host):
   ids = getid(host, user, m)
   r, alpha = pyoprf.blind(pwd)
 
@@ -548,25 +549,30 @@ def get(m, pwd, user, host, raw=False):
   m = ratelimit(m, msgs)
 
   if len(servers) > 1:
-    resps = m.gather(33+RULE_SIZE, threshold, lambda x: (x[:33], x[33:]))
+    resps = m.gather(33+RULE_SIZE, threshold)
     if resps is None:
       m.close()
-      raise ValueError("Failed to get answers from shareholders")
+      raise ValueError("Failed to get any answers from shareholders")
+    if len({x for x in resps if x is not None}) == 1 and {x for x in resps if x is not None} == {b'\x00\x04fail'}:
+      raise ValueError("ERROR: The record does not exist, there's a chance you are being fished.")
+    resps = [(x[:33], x[33:]) for x in resps if x is not None]
     if len({resp[1] for resp in resps if resp}) != 1:
       m.close()
       raise ValueError("ERROR: servers disagree on rules")
     rules = resps[0][1]
     beta = pyoprf.thresholdmult([resp[0] for resp in resps if resp])
   else:
-    resp = m.gather(32+RULE_SIZE, 1)[0] # beta + sealed rules
-    if resps is None:
+    resp = m.gather(33+RULE_SIZE, 1)[0] # beta + sealed rules
+    if resp is None:
       m.close()
       raise ValueError("Failed to get answers from sphinx server")
-    if resp == b'\x00\x04fail' or len(resp)!=32+RULE_SIZE:
+    if resp == b'\x00\x04fail':
+      raise ValueError("ERROR: The record does not exist, there's a chance you are being fished.")
+    if len(resp)!=33+RULE_SIZE:
       m.close()
-      raise ValueError("ERROR: Either the record does not exist, or the request to server was corrupted during transport.")
-    beta = resp[:32]
-    rules = resp[32:]
+      raise ValueError("ERROR: the request to server was corrupted during transport.")
+    beta = resp[1:33]
+    rules = resp[33:]
 
   rwd = pyoprf.unblind_finalize(r, beta, pwd)
 
@@ -582,7 +588,6 @@ def get(m, pwd, user, host, raw=False):
     raise ValueError("ERROR: bad checkdigit")
 
   rwd = xor(pysodium.crypto_generichash(PASS_CTX, rwd),xormask)
-  if raw == True: return rwd
 
   ret = convert(rwd,user,classes,size,symbols)
   clearmem(rwd)
@@ -655,11 +660,11 @@ def change(m, oldpwd, newpwd, user, host, classes='uld', symbols=bin2pass.symbol
     beta = dkg(m, CHANGE_DKG, threshold, None, alpha)
   else:
     m.broadcast(alpha)
-    beta = m.gather(32,1) # beta
-    if beta is None or len(beta[0])!=32:
+    beta = m.gather(33,1) # beta
+    if beta is None or len(beta[0])!=33:
       m.close()
       raise ValueError("ERROR: changing password failed due to corruption during transport.")
-    beta = beta[0]
+    beta = beta[0][1:]
   rwd = pyoprf.unblind_finalize(r, beta, newpwd)
 
   if validate_password:
@@ -799,10 +804,16 @@ def print_qr(qrcode: QrCode) -> None:
 
 def qrcode(output, key):
   mk=get_masterkey() if key else b''
-  data = (bytes([1*key+2*rwd_keys + 4*validate_password]) +
+  hosts = []
+  for name, server in servers.items():
+    if len(name)>255: raise ValueError(f"server name: {name} is too long, max 255 allowed in qr codes")
+    hosts.append(bytes([len(name.encode('utf8'))]) + name.encode('utf8') +
+                 bytes([len(server.get('host','localhost'))]) + server.get('host','localhost').encode('utf8') +
+                 struct.pack("!H", server.get('port', 2355)))
+  hosts=zlib.compress(b''.join(hosts))
+  data = (bytes([1*key+2*rwd_keys + 4*validate_password + 8*userlist, threshold]) +
           mk +
-          struct.pack("!H", port) +
-          hostname.encode("utf8"))
+          hosts)
 
   qr = QrCode.encode_binary(data, QrCode.Ecc.LOW)
   if key:
@@ -954,7 +965,7 @@ def main(params=sys.argv):
     except Exception as exc:
       error = exc
       ret = False
-      raise # only for dbg
+      #raise # only for dbg
     clearmem(pwd)
   else:
     try:
@@ -963,7 +974,7 @@ def main(params=sys.argv):
     except Exception as exc:
       error = exc
       ret = False
-      raise # only for dbg
+      #raise # only for dbg
   m.close()
 
   if not ret:
