@@ -107,9 +107,10 @@ class TestEndToEnd(unittest.TestCase):
           print(f"enabling byzantine peers {corrupt_dkg_lib}", file=log)
           env["BYZANTINE_DKG"]=corrupt_dkg_lib
         cls._oracles.append(
-          (subprocess.Popen("oracle", cwd = f"{root}/servers/{idx}/", stdout=log, stderr=log, pass_fds=[log.fileno()], env=env), log))
+          (subprocess.Popen(["python3", path.dirname(path.abspath(sphinx.__file__)) + "/oracle.py"], cwd = f"{root}/servers/{idx}/", stdout=log, stderr=log, pass_fds=[log.fileno()], env=env), log))
         log.close()
-      del env["BYZANTINE_DKG"]
+      if corrupt_dkg_lib is not None:
+        del env["BYZANTINE_DKG"]
       time.sleep(0.8)
 
     @classmethod
@@ -511,6 +512,22 @@ class TestEndToEndNoneEither(TestEndToEnd):
     sphinx.rwd_keys=True
     sphinx.userlist=True
 
+class TestEndToEndSingleMode(TestEndToEnd):
+  def setUp(self):
+    self.threshold=sphinx.threshold
+    sphinx.threshold=1
+    sphinx.servers=dict(list(sphinx.servers.items())[:1])
+    global N
+    self.N=N
+    N=1
+
+  def tearDown(self, *args, **kwargs):
+    super(TestEndToEndSingleMode, self).tearDown(*args, **kwargs)
+    sphinx.servers=orig_servers
+    sphinx.threshold=self.threshold
+    global N
+    N=self.N
+
 if __name__ == '__main__':
   unittest.main()
 
